@@ -664,27 +664,21 @@ class Transformers:
                                max_grad_norm=parameters.get('max_grad_norm'),
                                early_stopping_metric='eval_loss',
                                early_stopping_metric_minimize=True,
-                               sliding_window=True,
+                               sliding_window=False,
                                manual_seed=1234,
                                warmup_ratio=parameters.get('warmup_ratio'),
                                warmup_step=parameters.get('warmup_step'),
-                               #config=dict(attention_probs_dropout_prob=parameters.get('attention_probs_dropout_prob'),
-                               #            hidden_size=parameters.get('hidden_size'),
-                               #            hidden_dropout_prob=parameters.get('hidden_dropout_prob'),
-                               #            initializer_range=parameters.get('initializer_range'),
-                               #            layer_norm_eps=parameters.get('layer_norm_eps'),
-                               #            num_attention_heads=parameters.get('num_attention_heads'),
-                               #            num_hidden_layers=parameters.get('num_hidden_layers')
-                               #            ),
                                save_steps=2000,
                                logging_steps=100,
                                evaluate_during_training=True,
-                               eval_all_checkpoints=True,
+                               eval_all_checkpoints=False,
                                use_tensorboard=True,
                                overwrite_output_dir=True,
                                reprocess_input_data=True,
                                do_lower_case=True,
                                no_save=True,
+                               no_cache=False,
+                               silent=True,
                                best_model_dir=BEST_MODEL_DIR,
                                output_dir=OUTPUT_DIR,
                                cache_dir=CACHE_DIR,
